#
# File: CrossToolchain.cmake
#
# Date: Fri Mar 24 15:30:00 EST 2017
# Author: manuel.arguelles
#
# Copyright 2013 Smartmatic Corp. All Rights Reserved.
#
# Confidential Information of Smartmatic Labs. Not for disclosure or distribution
# prior written consent. This software contains code, techniques and know-how which
# is confidential and proprietary to Smartmatic.
#
# Use of this software is subject to the terms of an end user license agreement.
#
# This script defines the tool chain for cross compilation target.
#
# -DPACKAGE_ARCHITECTURE="i386"   for INTEL i386 cross compilation.
# -DPACKAGE_ARCHITECTURE="x86_64" for AMD64      cross compilation.
#
# Set TOOLCHAIN_SYSROOT envoriment variable to change default chroot path.
#

SET (CMAKE_SYSTEM_NAME Linux)
SET (CMAKE_SYSTEM_VERSION 1)

IF (DEFINED ENV{TOOLCHAIN_SYSROOT})
  SET (TOOLCHAIN_SYSROOT $ENV{TOOLCHAIN_SYSROOT} CACHE PATH "Cross compilation system root.")
ELSE ()
  SET (TOOLCHAIN_SYSROOT /srv/chroot/${BUILD_ARCHITECTURE} CACHE PATH "Cross compilation system root.")
  MESSAGE (STATUS "Enviroment variable TOOLCHAIN_SYSROOT not found. Using default value.")
ENDIF ()

IF (BUILD_ARCHITECTURE STREQUAL "i386" OR BUILD_ARCHITECTURE STREQUAL "x86_64")
  SET (BUILD_ARCHITECTURE ${BUILD_ARCHITECTURE} CACHE STRING "")
  SET (ARCHITECTURE_FULL_NAME "${BUILD_ARCHITECTURE}-linux-gnu" CACHE STRING "")

  SET (CMAKE_C_COMPILER   /usr/bin/gcc     CACHE PATH "" FORCE)
  SET (CMAKE_CXX_COMPILER /usr/bin/g++     CACHE PATH "" FORCE)
  SET (CMAKE_AR           /usr/bin/ar      CACHE PATH "" FORCE)
  SET (CMAKE_RANLIB       /usr/bin/ranlib  CACHE PATH "" FORCE)
  SET (CMAKE_LINKER       /usr/bin/ld      CACHE PATH "" FORCE)
  SET (CMAKE_OBJCOPY      /usr/bin/objcopy CACHE PATH "" FORCE)
  SET (CMAKE_STRIP        /usr/bin/strip   CACHE PATH "" FORCE)

  IF (BUILD_ARCHITECTURE STREQUAL "x86_64")
    SET (CMAKE_CXX_FLAGS "${CMAKE_CXX_FLAGS} -m64" CACHE STRING "")
    SET (CMAKE_C_FLAGS   "${CMAKE_C_FLAGS}   -m64" CACHE STRING "")
    SET (ARCHITECTURE_ALTERNATIVE_NAME "amd64")
  ELSE ()
    SET (CMAKE_CXX_FLAGS "${CMAKE_CXX_FLAGS} -m32" CACHE STRING "")
    SET (CMAKE_C_FLAGS   "${CMAKE_C_FLAGS}   -m32" CACHE STRING "")
    SET (ARCHITECTURE_ALTERNATIVE_NAME "i386")
    SET (SYSROOT_LIBRARIES //usr/lib32 ${TOOLCHAIN_SYSROOT}/usr/lib/${ARCHITECTURE_FULL_NAME} ${TOOLCHAIN_SYSROOT}/usr/lib CACHE STRING "")
  ENDIF ()

  SET (CMAKE_SHARED_LINKER_FLAGS "${CMAKE_SHARED_LINKER_FLAGS}" CACHE STRING "")
  SET (CMAKE_MODULE_LINKER_FLAGS "${CMAKE_MODULE_LINKER_FLAGS}" CACHE STRING "")
  SET (CMAKE_EXE_LINKER_FLAGS    "${CMAKE_EXE_LINKER_FLAGS}"    CACHE STRING "")
ENDIF ()

MESSAGE (STATUS "Cross compilation base path: ${TOOLCHAIN_SYSROOT}")

IF (NOT EXISTS ${TOOLCHAIN_SYSROOT})
  MESSAGE (FATAL_ERROR "Cross compilation base path not found!. Please set TOOLCHAIN_SYSROOT enviroment variable.")
ENDIF ()

SET (CMAKE_CXX_FLAGS "${CMAKE_CXX_FLAGS} -isystem ${TOOLCHAIN_SYSROOT}/usr/include/${ARCHITECTURE_FULL_NAME}")
SET (CMAKE_C_FLAGS   "${CMAKE_C_FLAGS}   -isystem ${TOOLCHAIN_SYSROOT}/usr/include/${ARCHITECTURE_FULL_NAME}")

SET (CMAKE_EXE_LINKER_FLAGS "${CMAKE_EXE_LINKER_FLAGS} -Wl,-rpath-link ${TOOLCHAIN_SYSROOT}/lib/")
SET (CMAKE_EXE_LINKER_FLAGS "${CMAKE_EXE_LINKER_FLAGS} -Wl,-rpath-link ${TOOLCHAIN_SYSROOT}/usr/lib/")
SET (CMAKE_EXE_LINKER_FLAGS "${CMAKE_EXE_LINKER_FLAGS} -Wl,-rpath-link ${TOOLCHAIN_SYSROOT}/lib/${ARCHITECTURE_FULL_NAME}")
SET (CMAKE_EXE_LINKER_FLAGS "${CMAKE_EXE_LINKER_FLAGS} -Wl,-rpath-link ${TOOLCHAIN_SYSROOT}/usr/lib/${ARCHITECTURE_FULL_NAME}")

SET (CMAKE_FIND_ROOT_PATH ${TOOLCHAIN_SYSROOT})
SET (SYSROOT_PKG_CONFIG_LIBDIR "${TOOLCHAIN_SYSROOT}/usr/lib/${ARCHITECTURE_FULL_NAME}/pkgconfig:${TOOLCHAIN_SYSROOT}/usr/lib/pkgconfig:${TOOLCHAIN_SYSROOT}/usr/share/pkgconfig" CACHE STRING "" FORCE)

SET (CMAKE_LIBRARY_PATH /lib/${ARCHITECTURE_FULL_NAME} /usr/lib/${ARCHITECTURE_FULL_NAME})

IF ("${CMAKE_BUILD_TYPE}" STREQUAL "Release")
  SET (CMAKE_SKIP_RPATH TRUE CACHE BOOL "")
ELSE ()
  SET (CMAKE_SKIP_RPATH FALSE CACHE BOOL "Runtime paths are added when using shared libraries.")
ENDIF ()

SET (CMAKE_FIND_ROOT_PATH_MODE_PROGRAM NEVER)
SET (CMAKE_FIND_ROOT_PATH_MODE_INCLUDE ONLY)
SET (CMAKE_FIND_ROOT_PATH_MODE_LIBRARY ONLY)
