#ifndef BEFORE_GENERATING_RESULTS_VERIFICATION_TEST
#define BEFORE_GENERATING_RESULTS_VERIFICATION_TEST

#ifdef ECUT_MAIN

#include <Configuration/s-a-e-s-functionality-conf-manager.h>
#include <cppunit/extensions/HelperMacros.h>
#include <Operation/Voting/VoteCompliance/VoteComplianceVerificator.hxx>
#include <tests/MockSaesFunctionalityConfigurationManager.hxx>
#include <Voting/PBReports/vote-results-schema.hxx>

using namespace Smartmatic::SAES::Functionality::Configuration;
using namespace Smartmatic::SAES::Operation::VoteCompliance;
using namespace Smartmatic::SAES::Voting::PBReports;

static const std::string CONTEST_CODE                                 = "147";
static const std::string PARTY_CODE                                   = "1712";

static const std::string PRINCIPAL_VOTE_BALLOT_OPTION_CODE            = "48359";
static const std::string STAND_IN_VOTE_BALLOT_OPTION_CODE             = "48364";

static const size_t      SUCCESS_PRINCIPAL_CANDIDATE_PREFERENCE_VOTES = 3;
static const size_t      SUCCESS_STAND_IN_CANDIDATE_PREFERENCE_VOTES  = 3;

static const size_t      FAILURE_PRINCIPAL_CANDIDATE_PREFERENCE_VOTES = 6;
static const size_t      FAILURE_STAND_IN_CANDIDATE_PREFERENCE_VOTES  = 6;

static const size_t      SUCCESS_PARTY_TOTAL_VOTES                    = 3;
static const size_t      FAILURE_PARTY_TOTAL_VOTES                    = 6;

static const size_t      SUCCESS_PARTY_LIST_VOTES                     = 3;
static const size_t      FAILURE_PARTY_LIST_VOTES                     = 6;

static const size_t      TOTAL_VOTES                                  = 5;

class BeforeGeneratingResultsVerificationTest : public CPPUNIT_NS::TestFixture
{
    public:

        BeforeGeneratingResultsVerificationTest ()
        {

        }

        void setUp()
        {
            MockSaesFunctionalityConfigurationManager configurationManager;
            configurationManager.initialize();
            SAESFunctionalityConfigurationManager::getInstance ( &configurationManager );

            m_candidatesSequence.clear();
            m_partiesSequence.clear();
            m_contestsSequence.clear();

            m_partyResults.setCode(PARTY_CODE);

            m_contestResults.setCode(CONTEST_CODE);
        }

        void tearDown()
        {
            VoteComplianceVerificator::getInstance()->dispose();
        }

        void checkTotalCumulativeSelections_partyVotesExceedTotalVotes_invalidVoteResults()
        {
            m_candidateResult = buildItemResult(PRINCIPAL_VOTE_BALLOT_OPTION_CODE, SUCCESS_PRINCIPAL_CANDIDATE_PREFERENCE_VOTES);
            m_candidatesSequence.push_back(m_candidateResult);
            m_itemsResults.setCandidate(m_candidatesSequence);
            m_partyResults.setTotal_votes(FAILURE_PARTY_TOTAL_VOTES);
            m_partyResults.setCandidateResults(m_itemsResults);

            m_candidatesSequence.clear();
            m_candidateResult = buildItemResult(STAND_IN_VOTE_BALLOT_OPTION_CODE, SUCCESS_STAND_IN_CANDIDATE_PREFERENCE_VOTES);
            m_candidatesSequence.push_back(m_candidateResult);
            m_itemsResults.setCandidate(m_candidatesSequence);
            m_partyResults.setSubstituteResults(m_itemsResults);

            m_partiesSequence.push_back(m_partyResults);
            m_contestResults.setPartyResults(m_partiesSequence);

            m_contestsSequence.push_back(m_contestResults);
            m_voteResults.setContestResults(m_contestsSequence);

            bool check = VoteComplianceVerificator::getInstance()->checkTotalCumulativeSelections(TOTAL_VOTES, m_voteResults);

            CPPUNIT_ASSERT_ASSERTION_FAIL( CPPUNIT_ASSERT( check ) );
        }

        void checkTotalCumulativeSelections_principalCandidateVotesExceedTotalVotes_invalidVoteResults()
        {
            m_candidateResult = buildItemResult(PRINCIPAL_VOTE_BALLOT_OPTION_CODE, FAILURE_PRINCIPAL_CANDIDATE_PREFERENCE_VOTES);
            m_candidatesSequence.push_back(m_candidateResult);
            m_itemsResults.setCandidate(m_candidatesSequence);
            m_partyResults.setTotal_votes(SUCCESS_PARTY_TOTAL_VOTES);
            m_partyResults.setCandidateResults(m_itemsResults);

            m_candidatesSequence.clear();
            m_candidateResult = buildItemResult(STAND_IN_VOTE_BALLOT_OPTION_CODE, SUCCESS_STAND_IN_CANDIDATE_PREFERENCE_VOTES);
            m_candidatesSequence.push_back(m_candidateResult);
            m_itemsResults.setCandidate(m_candidatesSequence);
            m_partyResults.setSubstituteResults(m_itemsResults);

            m_partiesSequence.push_back(m_partyResults);
            m_contestResults.setPartyResults(m_partiesSequence);

            m_contestsSequence.push_back(m_contestResults);
            m_voteResults.setContestResults(m_contestsSequence);

            bool check = VoteComplianceVerificator::getInstance()->checkTotalCumulativeSelections(TOTAL_VOTES, m_voteResults);

            CPPUNIT_ASSERT_ASSERTION_FAIL( CPPUNIT_ASSERT( check ) );
        }

        void checkTotalCumulativeSelections_standInCandidatesVotesExceedTotalVotes_invalidVoteResults()
        {
            m_candidateResult = buildItemResult(PRINCIPAL_VOTE_BALLOT_OPTION_CODE, SUCCESS_PRINCIPAL_CANDIDATE_PREFERENCE_VOTES);
            m_candidatesSequence.push_back(m_candidateResult);
            m_itemsResults.setCandidate(m_candidatesSequence);
            m_partyResults.setTotal_votes(SUCCESS_PARTY_TOTAL_VOTES);
            m_partyResults.setCandidateResults(m_itemsResults);

            m_candidatesSequence.clear();
            m_candidateResult = buildItemResult(STAND_IN_VOTE_BALLOT_OPTION_CODE, FAILURE_STAND_IN_CANDIDATE_PREFERENCE_VOTES);
            m_candidatesSequence.push_back(m_candidateResult);
            m_itemsResults.setCandidate(m_candidatesSequence);
            m_partyResults.setSubstituteResults(m_itemsResults);

            m_partiesSequence.push_back(m_partyResults);
            m_contestResults.setPartyResults(m_partiesSequence);

            m_contestsSequence.push_back(m_contestResults);
            m_voteResults.setContestResults(m_contestsSequence);

            bool check = VoteComplianceVerificator::getInstance()->checkTotalCumulativeSelections(TOTAL_VOTES, m_voteResults);

            CPPUNIT_ASSERT_ASSERTION_FAIL( CPPUNIT_ASSERT( check ) );
        }

        void checkTotalCumulativeSelections_categorizedVotesDoNotExceedTotalVotes_validVoteResults()
        {
            m_candidateResult = buildItemResult(PRINCIPAL_VOTE_BALLOT_OPTION_CODE, SUCCESS_PRINCIPAL_CANDIDATE_PREFERENCE_VOTES);
            m_candidatesSequence.push_back(m_candidateResult);
            m_itemsResults.setCandidate(m_candidatesSequence);
            m_partyResults.setTotal_votes(SUCCESS_PARTY_TOTAL_VOTES);
            m_partyResults.setCandidateResults(m_itemsResults);

            m_candidatesSequence.clear();
            m_candidateResult = buildItemResult(STAND_IN_VOTE_BALLOT_OPTION_CODE, SUCCESS_STAND_IN_CANDIDATE_PREFERENCE_VOTES);
            m_candidatesSequence.push_back(m_candidateResult);
            m_itemsResults.setCandidate(m_candidatesSequence);
            m_partyResults.setSubstituteResults(m_itemsResults);

            m_partiesSequence.push_back(m_partyResults);
            m_contestResults.setPartyResults(m_partiesSequence);

            m_contestsSequence.push_back(m_contestResults);
            m_voteResults.setContestResults(m_contestsSequence);

            bool check = VoteComplianceVerificator::getInstance()->checkTotalCumulativeSelections(TOTAL_VOTES, m_voteResults);

            CPPUNIT_ASSERT( check );
        }

        void checkIndividualCumulativeSelections_partyListVotesExceedPartyTotalVotes_invalidVoteResults()
        {
            m_candidateResult = buildItemResult(PRINCIPAL_VOTE_BALLOT_OPTION_CODE, FAILURE_PRINCIPAL_CANDIDATE_PREFERENCE_VOTES);
            m_candidatesSequence.push_back(m_candidateResult);
            m_itemsResults.setCandidate(m_candidatesSequence);
            m_partyResults.setList_votes(FAILURE_PARTY_LIST_VOTES);
            m_partyResults.setTotal_votes(SUCCESS_PARTY_TOTAL_VOTES);
            m_partyResults.setCandidateResults(m_itemsResults);

            m_candidatesSequence.clear();
            m_candidateResult = buildItemResult(STAND_IN_VOTE_BALLOT_OPTION_CODE, SUCCESS_STAND_IN_CANDIDATE_PREFERENCE_VOTES);
            m_candidatesSequence.push_back(m_candidateResult);
            m_itemsResults.setCandidate(m_candidatesSequence);
            m_partyResults.setSubstituteResults(m_itemsResults);

            m_partiesSequence.push_back(m_partyResults);
            m_contestResults.setPartyResults(m_partiesSequence);

            m_contestsSequence.push_back(m_contestResults);
            m_voteResults.setContestResults(m_contestsSequence);

            bool check = VoteComplianceVerificator::getInstance()->checkIndividualCumulativeSelections(m_voteResults);

            CPPUNIT_ASSERT_ASSERTION_FAIL( CPPUNIT_ASSERT( check ) );
        }

        void checkIndividualCumulativeSelections_principalCandidateVotesExceedPartyTotalVotes_invalidVoteResults()
        {
            m_candidateResult = buildItemResult(PRINCIPAL_VOTE_BALLOT_OPTION_CODE, FAILURE_PRINCIPAL_CANDIDATE_PREFERENCE_VOTES);
            m_candidatesSequence.push_back(m_candidateResult);
            m_itemsResults.setCandidate(m_candidatesSequence);
            m_partyResults.setList_votes(SUCCESS_PARTY_LIST_VOTES);
            m_partyResults.setTotal_votes(SUCCESS_PARTY_TOTAL_VOTES);
            m_partyResults.setCandidateResults(m_itemsResults);

            m_candidatesSequence.clear();
            m_candidateResult = buildItemResult(STAND_IN_VOTE_BALLOT_OPTION_CODE, SUCCESS_STAND_IN_CANDIDATE_PREFERENCE_VOTES);
            m_candidatesSequence.push_back(m_candidateResult);
            m_itemsResults.setCandidate(m_candidatesSequence);
            m_partyResults.setSubstituteResults(m_itemsResults);

            m_partiesSequence.push_back(m_partyResults);
            m_contestResults.setPartyResults(m_partiesSequence);

            m_contestsSequence.push_back(m_contestResults);
            m_voteResults.setContestResults(m_contestsSequence);

            bool check = VoteComplianceVerificator::getInstance()->checkIndividualCumulativeSelections(m_voteResults);

            CPPUNIT_ASSERT_ASSERTION_FAIL( CPPUNIT_ASSERT( check ) );
        }

        void checkIndividualCumulativeSelections_standInCandidateVotesExceedPartyTotalVotes_invalidVoteResults()
        {
            m_candidateResult = buildItemResult(PRINCIPAL_VOTE_BALLOT_OPTION_CODE, SUCCESS_PRINCIPAL_CANDIDATE_PREFERENCE_VOTES);
            m_candidatesSequence.push_back(m_candidateResult);
            m_itemsResults.setCandidate(m_candidatesSequence);
            m_partyResults.setList_votes(SUCCESS_PARTY_LIST_VOTES);
            m_partyResults.setTotal_votes(SUCCESS_PARTY_TOTAL_VOTES);
            m_partyResults.setCandidateResults(m_itemsResults);

            m_candidatesSequence.clear();
            m_candidateResult = buildItemResult(STAND_IN_VOTE_BALLOT_OPTION_CODE, FAILURE_STAND_IN_CANDIDATE_PREFERENCE_VOTES);
            m_candidatesSequence.push_back(m_candidateResult);
            m_itemsResults.setCandidate(m_candidatesSequence);
            m_partyResults.setSubstituteResults(m_itemsResults);

            m_partiesSequence.push_back(m_partyResults);
            m_contestResults.setPartyResults(m_partiesSequence);

            m_contestsSequence.push_back(m_contestResults);
            m_voteResults.setContestResults(m_contestsSequence);

            bool check = VoteComplianceVerificator::getInstance()->checkIndividualCumulativeSelections(m_voteResults);

            CPPUNIT_ASSERT_ASSERTION_FAIL( CPPUNIT_ASSERT( check ) );
        }

        void checkIndividualCumulativeSelections_categorizedVotesDoNotExceedPartyTotalVotes_validVoteResults()
        {
            m_candidateResult = buildItemResult(PRINCIPAL_VOTE_BALLOT_OPTION_CODE, SUCCESS_PRINCIPAL_CANDIDATE_PREFERENCE_VOTES);
            m_candidatesSequence.push_back(m_candidateResult);
            m_itemsResults.setCandidate(m_candidatesSequence);
            m_partyResults.setList_votes(SUCCESS_PARTY_LIST_VOTES);
            m_partyResults.setTotal_votes(SUCCESS_PARTY_TOTAL_VOTES);
            m_partyResults.setCandidateResults(m_itemsResults);

            m_candidatesSequence.clear();
            m_candidateResult = buildItemResult(STAND_IN_VOTE_BALLOT_OPTION_CODE, SUCCESS_STAND_IN_CANDIDATE_PREFERENCE_VOTES);
            m_candidatesSequence.push_back(m_candidateResult);
            m_itemsResults.setCandidate(m_candidatesSequence);
            m_partyResults.setSubstituteResults(m_itemsResults);

            m_partiesSequence.push_back(m_partyResults);
            m_contestResults.setPartyResults(m_partiesSequence);

            m_contestsSequence.push_back(m_contestResults);
            m_voteResults.setContestResults(m_contestsSequence);

            bool check = VoteComplianceVerificator::getInstance()->checkIndividualCumulativeSelections(m_voteResults);

            CPPUNIT_ASSERT( check );
        }

        CPPUNIT_TEST_SUITE ( BeforeGeneratingResultsVerificationTest );

        CPPUNIT_TEST ( checkTotalCumulativeSelections_partyVotesExceedTotalVotes_invalidVoteResults );
        CPPUNIT_TEST ( checkTotalCumulativeSelections_principalCandidateVotesExceedTotalVotes_invalidVoteResults );
        CPPUNIT_TEST ( checkTotalCumulativeSelections_standInCandidatesVotesExceedTotalVotes_invalidVoteResults );
        CPPUNIT_TEST ( checkTotalCumulativeSelections_categorizedVotesDoNotExceedTotalVotes_validVoteResults );

        CPPUNIT_TEST ( checkIndividualCumulativeSelections_partyListVotesExceedPartyTotalVotes_invalidVoteResults );
        CPPUNIT_TEST ( checkIndividualCumulativeSelections_principalCandidateVotesExceedPartyTotalVotes_invalidVoteResults );
        CPPUNIT_TEST ( checkIndividualCumulativeSelections_standInCandidateVotesExceedPartyTotalVotes_invalidVoteResults );
        CPPUNIT_TEST ( checkIndividualCumulativeSelections_categorizedVotesDoNotExceedPartyTotalVotes_validVoteResults );

        CPPUNIT_TEST_SUITE_END();

    private:

        Smartmatic::SAES::Voting::PBReports::ItemResult buildItemResult(
            const std::string &ballotOptionCode,
            const size_t preferenceVotes)
        {
            Smartmatic::SAES::Voting::PBReports::ItemResult candidateResult;
            candidateResult.setCode(ballotOptionCode);
            candidateResult.setPreference_votes(preferenceVotes);

            return candidateResult;
        }

        Smartmatic::SAES::Voting::PBReports::VoteResults m_voteResults;
        ItemsResults::CandidateSequence m_candidatesSequence;
        ItemResult m_candidateResult;
        ItemsResults m_itemsResults;
        PartyResults m_partyResults;
        ContestResults::PartyResultsSequence m_partiesSequence;
        VoteResults::ContestResultsType m_contestResults;
        VoteResults::ContestResultsSequence m_contestsSequence;
};

#endif /* ECUT_MAIN */

#endif /* BEFORE_GENERATING_RESULTS_VERIFICATION_TEST */
