/**
 * @file	CustomAttributeSelectionBoxWidget.hxx
 * @brief	Header custom attribute selection box widget class.
 * @author	Yadickson Soto
 * @date	13/11/2012
 */

#ifndef _CUSTOM_ATTRIBUTE_SELECTION_BOX_WIDGET_H_
#define _CUSTOM_ATTRIBUTE_SELECTION_BOX_WIDGET_H_

#include "ICustomAttributeSelectionWidget.hxx"
#include "CustomAttributeSelectionButtonWidget.hxx"
#include <Widgets/TableBoxSelectionWidget.hxx>
#include <Widgets/SmartmaticBasicLabel.hxx>
#include <Widgets/SmartmaticBasicVBox.hxx>
#include <Widgets/SmartmaticBasicFrame.hxx>

namespace Smartmatic
{
    namespace SAES
    {
        namespace GUI
        {
            namespace Widgets
            {
                namespace Voting
                {
                    /**
                     * @class	CustomAttributeSelectionBoxWidget
                     * @brief	Custom attribute selection box widget.
                     * @author	Yadickson Soto
                     * @date	13/11/2012
                     */
                    class CustomAttributeSelectionBoxWidget: public ICustomAttributeSelectionWidget
                    {
                        public:

                            /**
                             * @fn	CustomAttributeSelectionBoxWidget::CustomAttributeSelectionBoxWidget ( Smartmatic::SAES::Voting::VotingDevice::Custom_attribute_definition & customAttr,
                             * 			bool showConfirmation, bool sortAlphabetically );
                             * @brief	Constructor.
                             * @author	Yadickson Soto
                             * @date	13/11/2012
                             * @param	customAttr					The custom attribute.
                             * @param	showConfirmation		true to show, false to hide the confirmation.
                             * @param	sortAlphabetically	true to sort alphabetically.
                             */

                            CustomAttributeSelectionBoxWidget ( Smartmatic::SAES::Voting::VotingDevice::Custom_attribute_definition & customAttr,
                                                                bool showConfirmation, bool sortAlphabetically );

                            /**
                             * @fn	virtual CustomAttributeSelectionBoxWidget::~CustomAttributeSelectionBoxWidget();
                             * @brief	Finaliser.
                             * @author	Yadickson Soto
                             * @date	13/11/2012
                             */
                            virtual ~CustomAttributeSelectionBoxWidget();

                            /**
                             * @fn	virtual void CustomAttributeSelectionBoxWidget::ForceSelectedOption ( unsigned int index );
                             * @brief	Raises the answer event.
                             * @author	Yadickson Soto
                             * @date	13/11/2012
                             * @param	index index	to be selected.
                             */
                            virtual void ForceSelectedOption ( unsigned int index );
                            /**
                             * @fn	virtual void CustomAttributeSelectionBoxWidget::refresh();
                             * @brief	refresh widget method.
                             * @author	Yadickson Soto
                             * @date	13/11/2012
                             */
                            virtual void refresh();

                        private:

                            /**
                             * @fn	virtual void CustomAttributeSelectionBoxWidget::initialize();
                             * @brief	Initializer method.
                             * @author	Yadickson Soto
                             * @date	13/11/2012
                             */
                            virtual void initialize();
                            /**
                             * @fn	virtual void CustomAttributeSelectionBoxWidget::dispose();
                             * @brief	virtual method stop.
                             * @author	Yadickson Soto
                             * @date	13/11/2012
                             */
                            virtual void dispose();
                            /**
                             * @fn	void CustomAttributeSelectionBoxWidget::onAcceptClicked();
                             * @brief	method accept click button.
                             * @author	Yadickson Soto
                             * @date	14/11/2012
                             */
                            void onAcceptClicked();
                            /**
                             * @fn	void CustomAttributeSelectionBoxWidget::onClearClicked();
                             * @brief	method cancel click button.
                             * @author	Yadickson Soto
                             * @date	14/11/2012
                             */
                            void onClearClicked();
                            /**
                             * @fn	void CustomAttributeSelectionBoxWidget::onBoxClicked( std::list < Smartmatic::GUI::Widgets::BoxSelectionWidget * > & list);
                             * @brief	method box click.
                             * @author	Yadickson Soto
                             * @date	14/11/2012
                             * @param list box selected
                             */
                            void onBoxClicked ( std::list < Smartmatic::GUI::Widgets::BoxSelectionWidget * > & list );

                            Smartmatic::GUI::Widgets::SmartmaticBasicLabel * headerLabel; ///< header label
                            Smartmatic::GUI::Widgets::SmartmaticBasicFrame * bottomFrame; ///< bottom frame
                            Smartmatic::GUI::Widgets::SmartmaticBasicVBox * externalBox; ///< external vbox
                            CustomAttributeSelectionButtonWidget * buttonWidget; ///< button widget
                            Smartmatic::GUI::Widgets::BoxSelectionWidget * currentBox; ///< current box
                            Smartmatic::GUI::Widgets::TableBoxSelectionWidget * tableWidget; ///< table widgets
                    };
                }
            }
        }
    }
}

#endif // _CUSTOM_ATTRIBUTE_SELECTION_BOX_WIDGET_H_
