/**
 * @file	voting-widget-base.h
 * @brief	Declares the voting widget base class.
 * @author	Youssef.tourkmani
 * @date	01/11/2012
 */


#ifndef _VOTING_WIDGET_BASE_H_
#define _VOTING_WIDGET_BASE_H_

#include <gtkmm.h>
#include "Configuration/label-configuration.h"
#include <Configuration/SaesGUIConfiguration-schema.hxx>
#include <Widgets/SmartmaticBasicImage.hxx>
#include <Widgets/SmartmaticBasicLabel.hxx>

namespace Smartmatic
{
    namespace SAES
    {
        namespace GUI
        {
            namespace Widgets
            {
                /**
                 * @class	VotingWidgetBase
                 * @brief	Voting widget base.
                 * @author	Cesar.bonilla
                 * @date	4/29/2011
                 */

                class VotingWidgetBase: public Gtk::Container
                {
                    public:

                        /**
                         * @fn	VotingWidgetBase::VotingWidgetBase ( Smartmatic::SAES::Configuration::WidgetDisplayType & display,
                         * 			std::string displayString, std::string number, int padding, int numberWidth, int width,
                         * 			int height, bool showNumberA = true, bool isReferendum = false );
                         * @brief	class constructor.
                         * @author	Cesar.bonilla
                         * @date	4/29/2011
                         * @param	display			 	widget type.
                         * @param	displayString	if widget is Text is label name, if Image is image path.
                         * @param	number			 	number.
                         * @param	padding			 	widget padding.
                         * @param	numberWidth	 	Number of widths.
                         * @param	width				 	Widget's width.
                         * @param	height			 	Widget's height.
                         * @param	showNumberA	 	true to show, false to hide the number a.
                         * @param	isReferendum 	true to set referendum mode.
                         */

                        VotingWidgetBase ( Smartmatic::SAES::Configuration::WidgetDisplayType & display, std::string displayString, std::string number, int padding, int numberWidth, int width, int height, bool showNumberA = true, bool isReferendum = false );

                        /**
                         * @fn	VotingWidgetBase::VotingWidgetBase ( Smartmatic::SAES::Configuration::WidgetDisplayType & display,
                         * 			std::string displayString, std::string number, int padding, int numberWidth,
                         * 			Smartmatic::GUI::Configuration::LabelConfiguration labelChildConfigurationA, int width,
                         * 			int height, bool showNumberA = true );
                         * @brief	Constructor.
                         * @author	Cesar.bonilla
                         * @date	4/29/2011
                         * @param	display										The display.
                         * @param	displayString							The display string.
                         * @param	number										Number of.
                         * @param	padding										The padding.
                         * @param	numberWidth								Number of widths.
                         * @param	labelChildConfigurationA	The label child configuration a.
                         * @param	width											Widget's width.
                         * @param	height										Widget's height.
                         * @param	showNumberA								true to show, false to hide the number a.
                         */

                        VotingWidgetBase ( Smartmatic::SAES::Configuration::WidgetDisplayType & display, std::string displayString, std::string number, int padding, int numberWidth,
                                           Smartmatic::GUI::Configuration::LabelConfiguration labelChildConfigurationA, int width, int height, bool showNumberA = true );

                        /**
                         * @fn	VotingWidgetBase::VotingWidgetBase ( std::string topString, std::string bottomString,
                         * 			std::string number, int padding, int numberWidth, int width, int height,
                         * 			bool showNumberA = true );
                         * @brief	class constructor.
                         * @author	Cesar.bonilla
                         * @date	4/29/2011
                         * @param	topString			top string to be displayed.
                         * @param	bottomString	bottom string to be displayed.
                         * @param	number				number.
                         * @param	padding				widget padding.
                         * @param	numberWidth 	Number of widths.
                         * @param	width					Widget's width.
                         * @param	height				Widget's height.
                         * @param	showNumberA 	true to show, false to hide the number a.
                         */

                        VotingWidgetBase ( std::string topString, std::string bottomString, std::string number, int padding, int numberWidth, int width, int height, bool showNumberA = true );

                        /*void SetLabelConfiguration(Smartmatic::GUI::Configuration::LabelConfiguration labelChildConfigurationA,
                        							Smartmatic::GUI::Configuration::LabelConfiguration topChildConfigurationA,
                        							Smartmatic::GUI::Configuration::LabelConfiguration bottomChildConfigurationA,
                        							Smartmatic::GUI::Configuration::LabelConfiguration numberChildConfigurationA);

                        void SetLabelConfiguration(Smartmatic::GUI::Configuration::LabelConfiguration labelChildConfigurationA);*/

                        /**
                         * @fn	VotingWidgetBase::~VotingWidgetBase();
                         * @brief	class destructor.
                         * @author	Cesar.bonilla
                         * @date	4/29/2011
                         */

                        virtual ~VotingWidgetBase();

                        /**
                         * @fn	void VotingWidgetBase::Initialize ( int width, int height );
                         * @brief	create child widgets.
                         * @author	Cesar.bonilla
                         * @date	4/29/2011
                         * @param	width 	Widget's width.
                         * @param	height	Widget's height.
                         */

                        void Initialize ( int width, int height );

                        /**
                         * @fn	void VotingWidgetBase::setSize ( int width, int height );
                         * @brief	set size elements.
                         * @author	Yadickson Soto
                         * @date	16/03/2012
                         * @param	width 	Widget's width.
                         * @param	height	Widget's height.
                         */
                        void setSize ( int width, int height );

                        /**
                         * @fn	void VotingWidgetBase::refreshLabel ( std::string text );
                         * @brief	Refresh label.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         * @param	text	The text.
                         */
                        void refreshLabel ( std::string text );

                    protected:
                        /**
                         * Overrided virtual method from Gtk::Container
                         */
                        //virtual void on_size_request(Gtk::Requisition* requisition);

                        /**
                         * @fn	virtual void VotingWidgetBase::on_size_allocate ( Gtk::Allocation & allocation );
                         * @brief	Handles size allocate signals.
                         * @author	Cesar.bonilla
                         * @date	4/29/2011
                         * @param	allocation	The allocation.
                         */

                        virtual void on_size_allocate ( Gtk::Allocation & allocation );

                        /**
                         * @fn	virtual void VotingWidgetBase::forall_vfunc ( gboolean include_internals,
                         * 			GtkCallback callback, gpointer callback_data );
                         * @brief	Forall vfunc.
                         * @author	Cesar.bonilla
                         * @date	4/29/2011
                         * @param	include_internals	The include internals.
                         * @param	callback				 	The callback.
                         * @param	callback_data		 	Information describing the callback.
                         */

                        virtual void forall_vfunc ( gboolean include_internals, GtkCallback callback, gpointer callback_data );

                        /**
                         * @fn	virtual void VotingWidgetBase::on_add ( Gtk::Widget * child );
                         * @brief	Handles add signals.
                         * @author	Cesar.bonilla
                         * @date	4/29/2011
                         * @param	child	If non-null, the child.
                         */

                        virtual void on_add ( Gtk::Widget * child );

                        /**
                         * @fn	virtual void VotingWidgetBase::on_remove ( Gtk::Widget * child );
                         * @brief	Handles remove signals.
                         * @author	Cesar.bonilla
                         * @date	4/29/2011
                         * @param	child	If non-null, the child.
                         */

                        virtual void on_remove ( Gtk::Widget * child );

                        /**
                         * @fn	virtual GType VotingWidgetBase::child_type_vfunc() const;
                         * @brief	Gets the child type vfunc.
                         * @author	Cesar.bonilla
                         * @date	4/29/2011
                         * @return	.
                         */

                        virtual GType child_type_vfunc() const;

                    private:
                        /**
                         * Child Widgets
                         */
                        Smartmatic::GUI::Widgets::SmartmaticBasicImage * image_child;	///< The image child
                        Smartmatic::GUI::Widgets::SmartmaticBasicLabel * label_child;	///< The label child
                        Smartmatic::GUI::Widgets::SmartmaticBasicLabel * top_child; ///< The top child
                        Smartmatic::GUI::Widgets::SmartmaticBasicLabel * bottom_child;  ///< The bottom child
                        Smartmatic::GUI::Widgets::SmartmaticBasicLabel * number_child;  ///< Number of childs

                        /**
                         * Variables
                         */
                        bool showNumber;	///< true to show, false to hide the number
                        bool isReferendum;	///< true to show, false to hide the number
                        int numberWidth;	///< Number of widths
                        int padding;	///< The padding
                        bool sizeAllocateCallback;  ///< true to size allocate callback
                        Smartmatic::SAES::Configuration::WidgetDisplayType display;   ///< The display
                        std::string number; ///< Number
                        std::string displayString;  ///< The display string
                        std::string topString;  ///< The top string
                        std::string bottomString;   ///< The bottom string

                };
            }
        }
    }
}

#endif // _VOTING_WIDGET_BASE_H_
