/**
 * @file DetachedSignature.hxx
 * @brief Header class detached signature
 * @date 08/02/2013
 * @author Yadickson Soto
 */

#ifndef DETACHEDSIGNATURE_HXX_
#define DETACHEDSIGNATURE_HXX_

#include <System/SafeBuffer.h>

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }

    namespace System
    {
        namespace Security
        {
            /**
             * @class DetachedSignature
             * @brief class detached signature
             * @date 08/02/2013
             * @author Yadickson Soto
             */
            class DetachedSignature
            {
                public:

                    /**
                     * @brief Class constructor
                     * @date 08/02/2013
                     * @author Yadickson Soto
                     */
                    DetachedSignature();

                    /**
                     * @brief Class destructor
                     * @date 08/02/2013
                     * @author Yadickson Soto
                     */
                    virtual ~DetachedSignature();

                    /**
                     * @fn	bool verifyBuffer ( const Smartmatic::System::SafeBuffer & bufferIn, const std::string & basePath );
                     * @brief	method to validate detached xml resource list.
                     * @author	Yadickson Soto &lt;yadickson.soto@smartmatic.com&gt;
                     * @date	05/02/2014
                     * @param	bufferIn buffer detached xml
                     * @param	basePath base path resources.
                     * @return	true is validate resources.
                     */
                    bool verifyBuffer ( const Smartmatic::System::SafeBuffer & bufferIn,
                                        const std::string & basePath );

                private:

                    static Smartmatic::Log::ISMTTLog * logger; ///<logger
            };
        }
    }
}

#endif /* DETACHEDSIGNATURE_HXX_ */
