package com.smartmatic.security;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.OutputStream;
import java.io.Serializable;
import java.security.InvalidAlgorithmParameterException;
import java.security.InvalidKeyException;
import java.security.NoSuchAlgorithmException;
import java.security.spec.AlgorithmParameterSpec;
import javax.crypto.BadPaddingException;
import javax.crypto.Cipher;
import javax.crypto.CipherInputStream;
import javax.crypto.CipherOutputStream;
import javax.crypto.IllegalBlockSizeException;
import javax.crypto.NoSuchPaddingException;
import javax.crypto.SecretKey;
import javax.crypto.spec.IvParameterSpec;
import org.apache.commons.codec.binary.Base64;
import org.apache.commons.io.IOUtils;
import org.apache.commons.lang3.StringUtils;

public class PoolableSymmetricCipher
{
  public static final byte CBC = 0;
  public static final byte ECB = 1;
  protected static final String KEYALG = "AES";
  public static final String ALGORITHM_CBC = "AES/CBC/PKCS5Padding";
  public static final String ALGORITHM_ECB = "AES/ECB/PKCS5Padding";
  protected Cipher encodingCipher;
  protected Cipher decodingCipher;
  protected SecretKey secretKey;
  
  public static String encode(byte[] data)
  {
    return new String(Base64.encodeBase64(data, false));
  }
  
  public static String encode(byte[] data, String lineSeparator)
  {
    String encoded = new String(Base64.encodeBase64(data, false));
    
    String systemLineSeparator = System.getProperty("line.separator");
    if (!systemLineSeparator.equals(lineSeparator)) {
      encoded = StringUtils.replace(encoded, systemLineSeparator, 
        lineSeparator);
    }
    return encoded;
  }
  
  public static byte[] decode(String data)
  {
    return Base64.decodeBase64(data);
  }
  
  public PoolableSymmetricCipher(SecretKey key)
    throws NoSuchAlgorithmException, NoSuchPaddingException, InvalidKeyException
  {
    this.secretKey = key;
    
    this.decodingCipher = 
      Cipher.getInstance("AES/ECB/PKCS5Padding");
    this.encodingCipher = 
      Cipher.getInstance("AES/ECB/PKCS5Padding");
    
    this.encodingCipher.init(1, this.secretKey);
    this.decodingCipher.init(2, this.secretKey);
  }
  
  protected PoolableSymmetricCipher() {}
  
  public PoolableSymmetricCipher(SecretKey key, String iVector)
    throws NoSuchAlgorithmException, NoSuchPaddingException, InvalidKeyException, InvalidAlgorithmParameterException
  {
    this.secretKey = key;
    
    this.decodingCipher = 
      Cipher.getInstance("AES/CBC/PKCS5Padding");
    this.encodingCipher = 
      Cipher.getInstance("AES/CBC/PKCS5Padding");
    
    Base64 encoder = new Base64();
    byte[] iVectorData = encoder.decode(iVector.getBytes());
    
    AlgorithmParameterSpec parametersSpec = new IvParameterSpec(iVectorData);
    
    this.encodingCipher.init(1, this.secretKey, 
      parametersSpec);
    this.decodingCipher.init(2, this.secretKey, 
      parametersSpec);
  }
  
  public PoolableSymmetricCipher(SecretKey key, byte[] iVector)
    throws NoSuchAlgorithmException, NoSuchPaddingException, InvalidKeyException, InvalidAlgorithmParameterException
  {
    this.secretKey = key;
    
    this.decodingCipher = 
      Cipher.getInstance("AES/CBC/PKCS5Padding");
    this.encodingCipher = 
      Cipher.getInstance("AES/CBC/PKCS5Padding");
    
    byte[] iVectorData = iVector;
    
    AlgorithmParameterSpec parametersSpec = new IvParameterSpec(iVectorData);
    
    this.encodingCipher.init(1, this.secretKey, 
      parametersSpec);
    this.decodingCipher.init(2, this.secretKey, 
      parametersSpec);
  }
  
  public PoolableSymmetricCipher(SecretKey key, String iVector, String algorithm)
    throws NoSuchAlgorithmException, NoSuchPaddingException, InvalidKeyException, InvalidAlgorithmParameterException
  {
    this.secretKey = key;
    
    this.decodingCipher = Cipher.getInstance(algorithm);
    this.encodingCipher = Cipher.getInstance(algorithm);
    
    Base64 encoder = new Base64();
    byte[] iVectorData = encoder.decode(iVector.getBytes());
    
    AlgorithmParameterSpec parametersSpec = new IvParameterSpec(iVectorData);
    
    this.encodingCipher.init(1, this.secretKey, 
      parametersSpec);
    this.decodingCipher.init(2, this.secretKey, 
      parametersSpec);
  }
  
  public String encrypt(String message)
    throws IllegalBlockSizeException, BadPaddingException
  {
    if (message == null) {
      return message;
    }
    Base64 base64Encoder = new Base64();
    
    byte[] data = message.getBytes();
    String cipherText = new String(base64Encoder.encode(this.encodingCipher
      .doFinal(data)));
    return cipherText;
  }
  
  public CipherOutputStream encryptBytes(OutputStream messageDestination)
  {
    return new CipherOutputStream(messageDestination, this.encodingCipher);
  }
  
  public String decrypt(String message)
    throws IllegalBlockSizeException, BadPaddingException
  {
    Base64 base64Encoder = new Base64();
    byte[] dataDecoded = base64Encoder.decode(message.getBytes());
    return decrypt(dataDecoded);
  }
  
  public byte[] decryptBytes(String message)
    throws IllegalBlockSizeException, BadPaddingException
  {
    Base64 base64Encoder = new Base64();
    byte[] dataDecoded = base64Encoder.decode(message.getBytes());
    byte[] dataPlain = this.decodingCipher.doFinal(dataDecoded);
    return dataPlain;
  }
  
  public byte[] decryptBytes(byte[] encData)
    throws IllegalBlockSizeException, BadPaddingException
  {
    byte[] dataPlain = this.decodingCipher.doFinal(encData);
    return dataPlain;
  }
  
  public String decrypt(byte[] message)
    throws IllegalBlockSizeException, BadPaddingException
  {
    byte[] dataPlain = this.decodingCipher.doFinal(message);
    String plainText = new String(dataPlain);
    return plainText;
  }
  
  public CipherInputStream decrypt(InputStream message)
  {
    return new CipherInputStream(message, this.decodingCipher);
  }
  
  public byte[] encryptBytes(byte[] message)
    throws IllegalBlockSizeException, BadPaddingException
  {
    return this.encodingCipher.doFinal(message);
  }
  
  public String encryptBytesBase64(byte[] message)
    throws IllegalBlockSizeException, BadPaddingException
  {
    byte[] data = encryptBytes(message);
    Base64 base64Encoder = new Base64();
    
    String cipherText = new String(base64Encoder.encode(data));
    return cipherText;
  }
  
  public byte[] decryptToBytes(InputStream input)
    throws IOException
  {
    InputStream decryptedInput = new CipherInputStream(input, 
      this.decodingCipher);
    
    return IOUtils.toByteArray(decryptedInput);
  }
  
  public byte[] encryptObject(Serializable message)
    throws IOException, IllegalBlockSizeException, BadPaddingException
  {
    ByteArrayOutputStream aos = null;
    ObjectOutputStream oos = null;
    try
    {
      if (message == null) {
        return null;
      }
      aos = new ByteArrayOutputStream();
      oos = new ObjectOutputStream(aos);
      oos.writeObject(message);
      byte[] data = aos.toByteArray();
      
      byte[] cipherObject = this.encodingCipher.doFinal(data);
      
      return cipherObject;
    }
    finally
    {
      try
      {
        oos.close();
      }
      catch (IOException localIOException2) {}
    }
  }
  
  public Serializable decryptObject(byte[] object)
    throws IllegalBlockSizeException, BadPaddingException, IOException, ClassNotFoundException
  {
    ByteArrayInputStream bais = null;
    ObjectInputStream ois = null;
    byte[] dataPlain = null;
    try
    {
      dataPlain = this.decodingCipher.doFinal(object);
      
      bais = new ByteArrayInputStream(dataPlain);
      ois = new ObjectInputStream(bais);
      
      return (Serializable)ois.readObject();
    }
    finally
    {
      if (ois != null) {
        try
        {
          ois.close();
        }
        catch (IOException localIOException1) {}
      }
    }
  }
  
  public String encrypt(byte[] message)
    throws IllegalBlockSizeException, BadPaddingException
  {
    String cipherText = new String(
      encode(this.encodingCipher.doFinal(message)));
    return cipherText;
  }
}

