#include <iostream>
#include <config.h>
#include <Voting/VotingDevice/LogicalVoterGroupFactory.hxx>
#include <Voting/CardData/CardDataFactory.hxx>
#include <Operation/Devices/CardController.hxx>
#include <Card/P168ASmartcardReaderPowerManager.hxx>
#include <Security/KeyManager.hxx>
#include <Log/SMTTLogManager.h>
#include <Configuration/configuration-persistence-manager.h>
#include <Configuration/s-a-e-s-common-conf-manager.h>
#include <System/StackTraceManager.hxx>
#include <System/Security/OpenSSLSecurity.hxx>
#include <System/Security/XmlSecurityC.hxx>
#include <unistd.h>
#include "DefaultSaesCommonConfigManager.hxx"

using namespace Smartmatic::Configuration;
using namespace Smartmatic::Log;
using namespace Smartmatic::SAES::Operation::Devices;
using namespace Smartmatic::SAES::Common::Configuration;
using namespace Smartmatic::SAES::Voting::VotingDevice;
using namespace Smartmatic::SAES::Voting::CardData;
using namespace Smartmatic::SAES::Card;
using namespace Smartmatic::SAES::Common::Security;
using namespace Smartmatic::System;
using namespace Smartmatic::System::Security;

void test ( bool restart, bool user );

#define CONFIG_FILENAME "configuration-miscellaneous.xml" ///< config file base
#define LOG_CONFIG_FILENAME "log4-configuration.xml" ///< logger file

DefaultSaesCommonConfigurationManager configurationManager;

int main ( int argc, char ** argv )
{
    OpenSSLSecurity::init();
    XmlSecurityC::init();

    bool restart = false;
    bool user = false;
    bool config = false;
    bool security = false;
    bool logger = false;

    for ( int i = 1; i < argc; i++ )
    {
        if ( std::string ( argv [ i ] ) == "r" )
        {
            restart = true;
        }

        if ( std::string ( argv [ i ] ) == "u" )
        {
            user = true;
        }

        if ( std::string ( argv [ i ] ) == "c" )
        {
            config = true;
        }

        if ( std::string ( argv [ i ] ) == "s" )
        {
            security = true;
        }

        if ( std::string ( argv [ i ] ) == "l" )
        {
            logger = true;
        }
    }

    if (config)
    {
        ConfigurationPersistenceManager::filePath = CONFIG_FILENAME;
    }
    else
    {
        SAESCommonConfigurationManager::getInstance ( &configurationManager );
    }

    SAESCommonConfigurationManager::getInstance ()->getConfiguration()->getSmartcardConfiguration()->setSecureChannelEnabled ( security );

    StackTraceManager::getInstance()->start();

    if ( logger )
    {
        SMTTLogManager::getInstance()->setConfigFile ( LOG_CONFIG_FILENAME );
        SMTTLogManager::getInstance()->Reconfigure();

    }

    SafeBuffer key ( "12345678" );
    KeyManager::getInstance()->addKeyByType ( key, KeyType::CONFIGURATION_SECRET_KEY );

    test ( restart, user );

    return 0;
}

void test ( bool restart, bool user )
{
    CardController * cardController;
    cardController = CardController::getInstance ();

    Card_data cardData;
    Card_data cardDataOut;
    Logical_voter_group lvg;

    lvg = LogicalVoterGroupFactory::createLogicalVoterGroup();
    cardData = CardDataFactory::createCardData ( lvg, NULL );

    unsigned long count = 0;
    unsigned long errorCount = 0;

    cardController->open ( "" );

    bool showInsert = true;

    while ( true )
    {
        try
        {
            while ( !cardController->isCardPresent() )
            {
                if ( showInsert )
                {
                    std::cout << "Insert card" << std::endl;
                    showInsert = false;
                }

                sleep ( 1 );
            }

            showInsert = true;
            bool write, read;

            std::cout << "--------------------" << std::endl;
            std::cout << "Test (" << ++count << ") - " << errorCount << std::endl;

            write = cardController->writeCardData ( cardData );
            read = cardController->readCardData ( cardDataOut );

            if ( !write || !read )
            {
                errorCount++;
                std::cout << "Error Count (" << errorCount << ") - Test (" << count << ")" << std::endl;
                sleep ( 1 );
            }

            if ( user )
            {
                std::cout << "Remove card" << std::endl;

                while ( cardController->isCardPresent() )
                {
                    sleep ( 1 );
                }
            }

            if ( restart )
            {
                P168ASmartcardReaderPowerManager powerManager;

                powerManager.callRestartSmartcard();
            }
        }
        catch ( ... )
        {
            errorCount++;
            std::cout << "Error Count (" << errorCount << ") - Test (" << count << ")" << std::endl;
        }
    }

    cardController->close();
    std::cout << "-------------------"  << std::endl;
}
