package com.smartmatic.waffle.security.sign;

import static org.junit.Assert.assertEquals;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.security.PrivateKey;
import java.security.cert.X509Certificate;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.commons.io.FileUtils;
import org.apache.xml.security.signature.XMLSignature;
import org.junit.Before;
import org.junit.Test;

import com.smartmatic.waffle.security.asymmetric.AsymmetricKeyManager;

public class SignDetachedGeneratorTest {

	/** PKCS12 in bytes to generate the signature. */
	byte[] PKCS12Bytes;

	/** Text to sign. */
	String textToSign;

	/** The certificate password. */
	String caPassword;

	/** The certificate alias. */
	String caAlias;

	/** Signature algorithm. */
	String signatureAlgorithm;

	/** Files by base URI */
	Map<String, List<File>> filesByUriBase;

	/**
	 * Test SetUp method.
	 * 
	 * @throws java.lang.Exception
	 */
	@Before
	public void setUp() throws Exception {

		File pkcs12File = new File(
				"test/com/smartmatic/waffle/security/test/testResources/test_cert.p12");

		File fileToSign = new File(
				"test/com/smartmatic/waffle/security/test/testResources/xml_to_test_signing.xml");

		this.PKCS12Bytes = FileUtils.readFileToByteArray(pkcs12File);

		this.textToSign = FileUtils.readFileToString(fileToSign);

		this.caPassword = "bel2013";

		this.caAlias = "ecm";

		this.filesByUriBase = new HashMap<String, List<File>>();

		List<File> filesToSign = new ArrayList<File>();

		// filesToSign.add(pkcs12File);

		filesToSign.add(fileToSign);

		this.filesByUriBase.put("./", filesToSign);

		this.signatureAlgorithm = XMLSignature.ALGO_ID_SIGNATURE_RSA_SHA1;
	}

	/**
	 * Test method for
	 * {@link com.smartmatic.waffle.security.sign.SignDetachedGenerator#getSignedText()}
	 */
	@Test
	public void testGetSignedText() throws Exception {

		// Arrange
		PrivateKey certificateCA = AsymmetricKeyManager.getPrivateKey(
				new ByteArrayInputStream(this.PKCS12Bytes), this.caPassword,
				this.caAlias, this.caPassword);

		X509Certificate x509cert = AsymmetricKeyManager.getCertificate(
				new ByteArrayInputStream(this.PKCS12Bytes), this.caPassword,
				this.caAlias);

		SignDetachedGenerator generator = new SignDetachedGenerator(x509cert,
				certificateCA, this.signatureAlgorithm);

		for (String baseUri : this.filesByUriBase.keySet()) {
			generator.addReferences(baseUri, this.filesByUriBase.get(baseUri));
		}
		String textSigned = generator.getSignedText();

		// Act
		int signsChecked = SignValidator.validateSign(textSigned,
				x509cert.getPublicKey());

		// Assert
		assertEquals(signsChecked, 1);
	}

}
