package com.smartmatic.waffle.security.sign;

import static org.junit.Assert.assertNotNull;

import java.io.File;
import java.io.IOException;
import java.security.InvalidAlgorithmParameterException;
import java.security.NoSuchAlgorithmException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.xml.crypto.MarshalException;
import javax.xml.crypto.dsig.XMLSignatureException;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.TransformerException;

import org.apache.commons.io.FileUtils;
import org.apache.xml.security.signature.XMLSignature;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.junit.runners.Parameterized;
import org.xml.sax.SAXException;

import com.smartmatic.waffle.security.lang.CryptographicRuntimeException;
import com.smartmatic.waffle.security.lang.CryptographicStaticException;

@RunWith(Parameterized.class)
public class SignXMLUtilTest {

	private byte[] pkcs12;
	private File xmlFile;
	private String caPath;
	private String caAlias;
	private String caPassword;
	private String signatureAlgorithm;

	@Before
	public void setUp() {
		this.xmlFile = new File(
				"test/com/smartmatic/waffle/security/test/testResources/xml_to_test_signing.xml");
		File cert = new File(this.caPath);
		try {
			this.pkcs12 = FileUtils.readFileToByteArray(cert);
		} catch (IOException e) {
			e.printStackTrace();
		}

	}

	public SignXMLUtilTest(String caPath, String caAlias, String caPassword,
			String signatureAlgorithm) {
		this.caPath = caPath;
		this.caAlias = caAlias;
		this.caPassword = caPassword;
		this.signatureAlgorithm = signatureAlgorithm;
	}

	@Parameterized.Parameters
	public static Collection<String[]> signParameters() {
		return Arrays
				.asList(new String[][] {
						{
								"test/com/smartmatic/waffle/security/test/testResources/test_cert.p12",
								"ecm", "bel2013",
								XMLSignature.ALGO_ID_SIGNATURE_RSA_SHA1 },
						{
								"test/com/smartmatic/waffle/security/test/testResources/ec_test_cert.p12",
								"ECTest", "Test1234",
								XMLSignature.ALGO_ID_SIGNATURE_ECDSA_SHA1 }, });
	}

	@Test
	public void testSignXML() {
		StringBuilder xmlFileString = null;
		try {

			xmlFileString = new StringBuilder(FileUtils.readFileToString(
					this.xmlFile, "UTF-8"));

			SignXMLUtil.signXML(this.pkcs12, this.caAlias, this.caPassword,
					xmlFileString, this.signatureAlgorithm);

			assertNotNull(xmlFileString.toString());
		} catch (IOException e) {
			e.printStackTrace();
		} catch (CryptographicStaticException e) {
			e.printStackTrace();
		} catch (CryptographicRuntimeException e) {
			e.printStackTrace();
		} catch (NoSuchAlgorithmException e) {
			e.printStackTrace();
		} catch (InvalidAlgorithmParameterException e) {
			e.printStackTrace();
		} catch (SAXException e) {
			e.printStackTrace();
		} catch (ParserConfigurationException e) {
			e.printStackTrace();
		} catch (MarshalException e) {
			e.printStackTrace();
		} catch (XMLSignatureException e) {
			e.printStackTrace();
		} catch (TransformerException e) {
			e.printStackTrace();
		}

	}

	@Test
	public void testSignDetachedXML() {
		Map<String, List<File>> filesByUriBase = new HashMap<String, List<File>>();
		List<File> files = new ArrayList<File>();
		files.add(this.xmlFile);
		filesByUriBase.put("./", files);
		String rs = null;
		try {
			rs = SignXMLUtil.signDetachedXML(this.pkcs12, this.caAlias,
					this.caPassword, filesByUriBase, this.signatureAlgorithm);

			assertNotNull(rs);
		} catch (IOException e) {
			e.printStackTrace();
		} catch (CryptographicStaticException e) {
			e.printStackTrace();
		} catch (CryptographicRuntimeException e) {
			e.printStackTrace();
		} catch (NoSuchAlgorithmException e) {
			e.printStackTrace();
		} catch (InvalidAlgorithmParameterException e) {
			e.printStackTrace();
		} catch (SAXException e) {
			e.printStackTrace();
		} catch (ParserConfigurationException e) {
			e.printStackTrace();
		} catch (MarshalException e) {
			e.printStackTrace();
		} catch (XMLSignatureException e) {
			e.printStackTrace();
		} catch (TransformerException e) {
			e.printStackTrace();
		}

	}
}
